package com.ruoyi.web.controller.sensor;

import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.sm4.SM4Utils;
import com.ruoyi.sensor.domain.PerceptionParams;
import com.ruoyi.sensor.service.ISensorDataService;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@RestController
@RequestMapping("/sensor/data")
public class OpenSensorDataController {

    private static final String SIGN_HEADER_NAME = "sign";
    private static final String SIGN_KEY_BASE64 = "RkYnZj5lcChmMSZQQHJFUg==";

    private static final Logger log = LoggerFactory.getLogger(OpenSensorDataController.class);

    @Autowired
    private ISensorDataService sensorDataService;

    @Value("${remote.sign}")
    private boolean signSwitch;


    @Anonymous
    @PostMapping("/judge")
    public AjaxResult list(HttpServletRequest request, @RequestBody PerceptionParams perceptionParams) {
        if (signSwitch) {
            if (!verifyRequest(request, perceptionParams.getStr())) {
                return AjaxResult.error("sign in error");
            }
        }
        sensorDataService.putPerceptionParamsToMap(perceptionParams);
        return AjaxResult.success();
    }

    private boolean verifyRequest(HttpServletRequest request, String str) {
        byte[] keyByte = Base64.decodeBase64(SIGN_KEY_BASE64);
        String sign = request.getHeader(SIGN_HEADER_NAME);
        if (StringUtils.isEmpty(sign)) {
            log.error("Header 中没有 sign");
            return false;
        }
        // 验证是否正确
        try {
            byte[] bytes = SM4Utils.encryptData_ECB(str, keyByte);
            String base64String = Base64.encodeBase64String(bytes);
            log.info("sign:{}, base64String:{}", sign, base64String);
            return sign.equals(base64String);
        } catch (Exception e) {
            log.error("{}", e.getMessage());
        }
        return false;
    }
}
